/*
	(C) 1995-99 AROS - The Amiga Research OS
	$Id: hidewindow.c,v 1.5 2009/04/05 16:39:33 jacadcaps Exp $
 
	Desc: Intuition function HideWindow()
	Lang: english
*/
#include <proto/layers.h>
#include "showhide.h"
#include "intuition_intern.h"
#include "intuition_private.h"
#include "inputhandler_actions.h"
#include "mosmisc.h"

#ifdef CGXSHOWHIDESUPPORT
#undef ChangeLayerVisibility
struct HideWindowActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
};

static VOID int_hidewindow(struct HideWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase);
#endif

#ifdef ChangeLayerVisibility
struct HideWindowActionMsg
{
	struct IntuiActionMsg msg;
	struct Window *window;
};

static VOID int_hidewindow(struct HideWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase);
#endif

/*****************************************************************************
 
	NAME */
#include <proto/intuition.h>

AROS_LH1(VOID, HideWindow,

		 /*  SYNOPSIS */
		 AROS_LHA(struct Window *, window, A0),

		 /*  LOCATION */
		 struct IntuitionBase *, IntuitionBase, 141, Intuition)

/*  FUNCTION
		Make a window invisible.
 
	INPUTS
	window - The window to affect.
 
	RESULT
 
	NOTES
 
	EXAMPLE
 
	BUGS
 
	SEE ALSO
 
	INTERNALS
 
	HISTORY
 
*****************************************************************************/
{
	AROS_LIBFUNC_INIT
	AROS_LIBBASE_EXT_DECL(struct IntuitionBase *,IntuitionBase)

#ifdef CGXSHOWHIDESUPPORT
	struct HideWindowActionMsg msg;

	DEBUG_HIDEWINDOW(dprintf("HideWindow: Window 0x%lx\n", (ULONG) window));

	SANITY_CHECK(window)
	if (window->Flags & WFLG_BACKDROP) return;

	msg.window = window;
	DoASyncAction((APTR)int_hidewindow, &msg.msg, sizeof(msg), IntuitionBase);
#endif

#ifdef ChangeLayerVisibility
	struct HideWindowActionMsg msg;

	DEBUG_HIDEWINDOW(dprintf("HideWindow: Window 0x%lx\n", window));

	msg.window = window;
	DoASyncAction((APTR)int_hidewindow, &msg.msg, sizeof(msg), IntuitionBase);
#endif

	AROS_LIBFUNC_EXIT
} /* HideWindow */


#ifdef CGXSHOWHIDESUPPORT
static VOID int_hidewindow(struct HideWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;

	if (!window) return;
	if (window->Flags & WFLG_BACKDROP) return;

	if (((struct IntWindow *)(window))->specialflags & SPFLAG_NOICONIFY) return;

	if (((struct Library *)LayersBase)->lib_Version < 52)
	{
		if (!CGXSystemBase) return;
		CGXHideWindow(window);
	}
	else
	{
		LayersControlTags(&window->WScreen->LayerInfo,
								LT_Layer,(ULONG)WLAYER(window),
								LT_Opacity,0,
								(BLAYER(window) != WLAYER(window)) ? LT_Layer : TAG_IGNORE,(ULONG)BLAYER(window),
								(BLAYER(window) != WLAYER(window)) ? LT_Opacity : TAG_IGNORE,0,
								TAG_DONE);
	}

	((struct IntWindow *)(window))->specialflags |= SPFLAG_ICONIFIED;
	CheckMouseZone(FALSE,IntuitionBase);
	UpdateScreenPointer(window->WScreen,IntuitionBase);

	if (IntuitionBase->ActiveWindow == window)
	{
		DeactivateWindow(window,NULL,IntuitionBase);
	}
};
#endif

#ifdef ChangeLayerVisibility
static VOID int_hidewindow(struct HideWindowActionMsg *msg,
						   struct IntuitionBase *IntuitionBase)
{
	struct Window *window = msg->window;
	struct Screen *screen = window->WScreen;

	if (!IsWindowVisible(window))
	{
		struct Requester *req;

		LOCK_REFRESH(screen);

		if (BLAYER(window))
		{
			ChangeLayerVisibility(BLAYER(window), FALSE);
		}
		ChangeLayerVisibility(WLAYER(window), FALSE);

		for (req = window->FirstRequest; req; req = req->OlderRequest)
		{
			ChangeLayerVisibility(req->ReqLayer, FALSE);
		}

		UNLOCK_REFRESH(screen);

		CheckLayers(screen, IntuitionBase);
	}
}
#endif
